@@ -1,8 +1,8 @@
 # Vend::Dispatch - Handle Interchange page requests
 #
-# $Id: Dispatch.pm,v 1.36.2.1 2004/04/19 14:32:59 mheins Exp $
+# $Id: Dispatch.pm,v 1.60 2005/10/31 15:13:09 mheins Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 2002 Mike Heins <mike@perusion.net>
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -26,7 +26,7 @@
 package Vend::Dispatch;
 
 use vars qw($VERSION);
-$VERSION = substr(q$Revision: 1.36.2.1 $, 10);
+$VERSION = substr(q$Revision: 1.60 $, 10);
 
 use POSIX qw(strftime);
 use Vend::Util;
@@ -39,6 +39,7 @@ use Vend::Session;
 use Vend::Page;
 use Vend::UserDB;
 use Vend::CounterFile;
+no warnings qw(uninitialized numeric);
 
 # TRACK
 use Vend::Track;
@@ -61,6 +62,8 @@ require Exporter;
 				update_values
 			);
 
+use strict;
+
 my $H;
 sub http {
 	return $H;
@@ -69,12 +72,9 @@ sub http {
 sub response {
 	my $possible = shift;
 	return if $Vend::Sent;
-	if($::Pragma->{download}) {
-		my $out = ref $possible ? $$possible : $possible;
-		# do nothing
-	}
-	elsif(defined $possible) {
-		push @Vend::Output, ( ref $possible ? $possible : \$possible);
+
+	if (defined $possible and ! $::Pragma->{download}) {
+		push @Vend::Output, (ref $possible ? $possible : \$possible);
 	}
 
 	if($::Pragma->{download}) {
@@ -342,10 +342,11 @@ sub do_deliver {
 	{
 		$Vend::StatusLine = "Status: 403\nContent-Type: text/html";
 		my $msg = get_locale_message(403, <<EOF);
-<B>Authorization Required<B>
-<P>
+<b>Authorization Required</b>
+<p>
 This server could not verify that you are authorized to access the document
 requested. 
+</p>
 EOF
 		response($msg);
 		return 0;
@@ -354,10 +355,10 @@ EOF
 	if (! -f $file) {
 		$Vend::StatusLine = "Status: 404\nContent-Type: text/html";
 		my $msg = get_locale_message(404, <<EOF, $file);
-<B>Not Found<B>
-<P>
+<b>Not Found</b>
+<p>
 The requested file %s was not found on this server.
-
+</p>
 EOF
 		response($msg);
 		return 0;
@@ -577,7 +578,33 @@ $form_action{go} = $form_action{return};
 
 sub do_process {
 
-    my @filters = grep /^[mu][vi]_filter:/, keys %CGI::values;
+	# Prevent using keys operation more than once
+    my @cgikeys = keys %CGI::values;
+
+    my @multis = grep /^mv\d\d?_/, @cgikeys;
+
+	## Only operates on up to 100 items to prevent "amplification"
+	## which could result in DOS attacks
+	MULTIS:
+	if(@multis) {
+		my %hash;
+		for(@multis) {
+			my $val = delete $CGI::values{$_};
+			# Have to handle nulls somehow....
+			$val =~ s/\0/::/g;
+			m{^mv\d+\d?_(.*)};
+			my $idx = $1;
+			my $key = $2;
+			$hash{$key} ||= [];
+			$hash{$key}[$idx] = $val;
+		}
+		while (my ($k, $v) = each %hash) {
+			$CGI::values{$k} = join "\0", @$v;
+		}
+	}
+
+    my @filters = grep /^[mu][vi]_filter:/, @cgikeys;
+
 	FILTERS: {
 		last FILTERS unless @filters;
 		foreach my $key (@filters) {
@@ -623,7 +650,7 @@ sub do_process {
 		# by Jeff Carnahan
 		$todo = action_map($x,$y,$map);
 	}
-	elsif( my @todo = grep /^mv_todo\.\w+(?:\.x)?$/, keys %CGI::values ) {
+	elsif( my @todo = grep /^mv_todo\.\w+(?:\.x)?$/, @cgikeys ) {
 		# Only one todo!
 		for(@todo) {
 			delete $CGI::values{$_};
@@ -642,8 +669,10 @@ sub do_process {
 		$sub = $Vend::Cfg->{FormAction}{$todo};
 	}
     elsif (not $sub = $form_action{$todo} ) {
+		unless ($sub = Vend::Util::codedef_routine('FormAction', $todo)) {
 		interaction_error("No action passed for processing\n");
 		return;
+    }
     }
 	eval {
 		$status = $sub->($todo);
@@ -748,6 +777,19 @@ sub run_in_catalog {
 	undef $Vend::Session;
 	
 	if(@itl) {
+		# Track job
+		my ($trackdb, $trackid);
+		
+		if ($jobscfg->{trackdb}) {
+			if ($trackdb = database_exists_ref($jobscfg->{trackdb})) {
+				$trackid = $trackdb->set_slice('', [qw(name begin_run pid)],
+											   [$job, Vend::Interpolate::mvtime(undef, {}, '%Y-%m-%d %H:%M'), $$]);
+			}
+			else {
+				::logError ("Invalid jobs tracking database $jobscfg->{trackdb}");
+			}
+		}
+		
 		# Run once at beginning
 		run_macro($jobscfg->{initialize});
 
@@ -766,6 +808,14 @@ sub run_in_catalog {
 			run_macro($jobscfg->{autoload});
 
 			push @out, interpolate_html($_->[1]);
+
+			# Run once at end of each job
+			run_macro($jobscfg->{autoend});
+		}
+
+		if ($trackid) {
+			$trackdb->set_field($trackid, 'end_run',
+								Vend::Interpolate::mvtime(undef, {}, '%Y-%m-%d %H:%M'));
 		}
 	}
 	else {
@@ -995,8 +1045,16 @@ EOF
 		}
 	}
 
-	$::Variable = $Vend::Cfg->{Variable};
-	$::Pragma   = { %{ $Vend::Cfg->{Pragma} } };
+	if ($Global::Foreground) {
+		my %hash;
+		tie %hash, 'Tie::ShadowHash', $Vend::Cfg->{Variable};
+		$::Variable = \%hash;
+		$::Pragma = { %{ $Vend::Cfg->{Pragma} } };
+	}
+	else {
+		$::Variable = $Vend::Cfg->{Variable};
+		$::Pragma = $Vend::Cfg->{Pragma};
+	}
 
 	my $mt;
 	if($Vend::Cfg->{DeliverImage}
@@ -1007,10 +1065,12 @@ EOF
 	  )
 	{
 
-#::logDebug("deliver image: method=$CGI::request_method type=$mt");
 		my $imgdir = $Vend::Cfg->{ImageDir};
 		my $fn = $CGI::path_info;
+#::logDebug("deliver image: method=$CGI::request_method type=$mt fn=$fn");
 		$fn =~ s:^/+::;
+		## Won't resend any images beginning with admin/
+		$fn =~ s{^admin/}{};
 		if($CGI::secure) {
 			 $imgdir = $Vend::Cfg->{ImageDirSecure}
 				if $Vend::Cfg->{ImageDirSecure};
@@ -1053,6 +1113,15 @@ EOF
 		}
 	}
 
+	if($Vend::Cfg->{XHTML}) {
+		$Vend::Xtrailer = '/';
+		$Vend::Xquote = '"';
+	}
+	else {
+		$Vend::Xtrailer = '';
+		$Vend::Xquote = '';
+	}
+
 	chdir $Vend::Cfg->{VendRoot} 
 		or die "Couldn't change to $Vend::Cfg->{VendRoot}: $!\n";
 	POSIX::setlocale(POSIX::LC_ALL, $Vend::Cfg->{ExecutionLocale});
@@ -1091,7 +1160,15 @@ sub run_macro {
 		if ($m =~ /^\w+$/) {
 			my $sub = $Vend::Cfg->{Sub}{$m} || $Global::GlobalSub->{$m}
 				or do {
-					logError("Unknown Autoload macro '%s'.", $macro);
+					my $call = join(',', caller());
+
+					my $msg = errmsg("Unknown macro '%s' from %s.", $m, $call);
+					if($Vend::Cfg->{CatalogName}) {
+						logError($msg);
+					}
+					else {
+						logGlobal($msg);
+					}
 					next;
 				};
 			$sub->($content_ref);
@@ -1119,8 +1196,8 @@ sub dispatch {
 
     my($sessionid, $seed);
 
-	$sessionid = $CGI::values{mv_session_id} || undef;
-	$sessionid =~ s/\0.*//s;
+	$sessionid = $CGI::values{mv_session_id} || undef
+		and $sessionid =~ s/\0.*//s;
 
 	$::Instance->{CookieName} = $Vend::Cfg->{CookieName};
 
@@ -1132,6 +1209,9 @@ sub dispatch {
 		$Vend::Cfg->{ScratchDefault}{mv_no_count} = 1;
 		$Vend::Cfg->{ScratchDefault}{mv_no_session_id} = 1;
 	}
+	elsif ($sessionid and $CGI::values{mv_force_session}) {
+		# do nothing
+	}
 	elsif ($::Instance->{CookieName} and defined $CGI::cookie) {
 		$CGI::cookie =~ m{$::Instance->{CookieName}=($Vend::Cfg->{CookiePattern})};
 		$seed = $sessionid = $1;
@@ -1263,7 +1343,8 @@ RESOLVEID: {
 			"WARNING: POSSIBLE BAD ROBOT. %s accesses with no 30 second pause.",
 			$Vend::Session->{accesses},
 					);
-					do_lockout($msg);
+					::logError($msg);
+					do_lockout();
 				}
 			}
 		}
@@ -1273,14 +1354,17 @@ RESOLVEID: {
 			if (Vend::Session::count_ip() > $Vend::Cfg->{RobotLimit}) {
 				my $msg;
 				# Here they can get it back if they pass expiration time
-				my $wait = $Global::Variable->{MV_ROBOT_EXPIRE} || 86400;
-				$wait /= 3600;
+				my $wait = $Vend::Cfg->{Limit}{robot_expire} || 1;
+				$wait *= 24;
 				$msg = errmsg(<<EOF, $wait); 
 Too many new ID assignments for this IP address. Please wait at least %d hours
 before trying again. Only waiting that period will allow access. Terminating.
 EOF
-				$msg = Vend::Page::get_locale_message(403, $msg);
-				do_lockout($msg);
+				$msg = get_locale_message(403, $msg);
+				do_lockout();
+
+				::logError('Too many IDs, %d hour wait enforced.', $wait);
+
 				$Vend::StatusLine = <<EOF;
 Status: 403 Forbidden
 Content-Type: text/plain
@@ -1299,38 +1383,13 @@ EOF
 	$Vend::Calc_initialized = 0;
 	$CGI::values{mv_session_id} = $Vend::Session->{id} = $Vend::SessionID;
 	if(my $vspace = $CGI::values{mv_values_space}) {
-		$Vend::Session->{values_repository} ||= {};
 		$::Values = $Vend::Session->{values_repository}{$vspace} ||= {};
-	}
-
-	if($Vend::Cfg->{CookieLogin} and ! $Vend::Session->{logged_in}) {
-		COOKIELOGIN: {
-			my $username;
-			my $password;
-			last COOKIELOGIN
-				if  exists  $CGI::values{mv_username}
-				and defined $CGI::values{mv_username};
-			last COOKIELOGIN
-				unless $username = Vend::Util::read_cookie('MV_USERNAME');
-			last COOKIELOGIN
-				unless $password = Vend::Util::read_cookie('MV_PASSWORD');
-			$CGI::values{mv_username} = $username;
-			$CGI::values{mv_password} = $password;
-			my $profile = Vend::Util::read_cookie('MV_USERPROFILE');
-			local(%SIG);
-			undef $SIG{__DIE__};
-			eval {
-				Vend::UserDB::userdb('login', profile => $profile );
-			};
-			if($@) {
-				$Vend::Session->{failure} .= $@;
-			}
-		}
+		$Vend::ValuesSpace = $vspace;
 	}
 
 	$Vend::Session->{'arg'} = $Vend::Argument = ($CGI::values{mv_arg} || undef);
 
-	if ($CGI::values{mv_pc} =~ /\D/) {
+	if ($CGI::values{mv_pc} and $CGI::values{mv_pc} =~ /\D/) {
 		$Vend::Session->{source} =	$CGI::values{mv_pc} eq 'RESET'
 										? ''
 										: $CGI::values{mv_pc};
@@ -1369,10 +1428,10 @@ EOF
 							my $msg = shift;
 							put_session() if $Vend::HaveSession;
 							my $content = get_locale_message(500, <<EOF, $msg);
-<HTML><HEAD><TITLE>Fatal Interchange Error</TITLE></HEAD><BODY>
-<H1>FATAL error</H1>
-<PRE>%s</PRE>
-</BODY></HTML>
+<html><head><title>Fatal Interchange Error</title></head><body>
+<h1>FATAL error</h1>
+<pre>%s</pre>
+</body></html>
 EOF
 							response(\$content);
 							exit 0;
@@ -1391,31 +1450,20 @@ EOF
 		}
 		$Vend::FinalPath =~ s{/$Vend::Cfg->{ProcessPage}/page/}{/};
 	}
-	my $locale;
-	if($locale = $::Scratch->{mv_language}) {
+
+	if(my $locale = $::Scratch->{mv_language}) {
 		$Global::Variable->{LANG}
 			= $::Variable->{LANG} = $locale;
 	}
-
-	if ($Vend::Cfg->{Locale}								and
-		$locale = $::Scratch->{mv_locale}	and
-		defined $Vend::Cfg->{Locale_repository}->{$locale}
-		)
-	{ 
-		$Global::Variable->{LANG}
-				= $::Variable->{LANG}
-				= $::Scratch->{mv_language}
-				= $locale
-			 if ! $::Scratch->{mv_language};
-		Vend::Util::setlocale(	$locale,
-								($::Scratch->{mv_currency} || undef),
-								{ persist => 1 }
-							);
-	}
 # END LEGACY
 
-	run_macro($Vend::Cfg->{Autoload});
-#show_times("end global Autoload macro") if $Global::ShowTimes;
+	if(my $ary = $Vend::Cfg->{DispatchRoutines}) {
+		for(@$ary) {
+			$_->();
+		}
+	}
+
+#show_times("end dispatch routines (Autoload, etc.)") if $Global::ShowTimes;
 
 	for my $macro ( $Vend::Cfg->{Filter}, $Vend::Session->{Filter}) {
 		next unless $macro;
@@ -1427,6 +1475,37 @@ EOF
 		}
 	}
 
+	## Here we initialize new features
+	if(my $ary = $Vend::Cfg->{Init}) {
+		undef $Vend::Cfg->{Init};
+		for(@$ary) {
+			my ($source, $touch) = @$_;
+			next if -f $touch;
+			open INITOUT, "> $touch"
+				or do {
+					::logError(
+						"Unable to open init file %s for feature init", $touch,
+						);
+					next;
+				};
+			my $out;
+			eval {
+				$out = Vend::Interpolate::interpolate_html(
+									Vend::Util::readfile($source)
+						  );
+			};
+			if($@) {
+				$out .= $@;
+			}
+			print INITOUT errmsg(
+							"Results of init at %s: ",
+							POSIX::strftime("%Y-%m-%d %H:%M:%S", localtime()),
+							);
+			print INITOUT $out;
+			close INITOUT;
+		}
+	}
+
 	run_macro($Vend::Session->{Autoload});
 #show_times("end session Autoload macro") if $Global::ShowTimes;
 
@@ -1561,13 +1640,7 @@ EOF
 #show_times("end page display") if $Global::ShowTimes;
 
 	if(my $macro = $Vend::Cfg->{AutoEnd}) {
-		if($macro =~ /\[\w+/) {
-			interpolate_html($macro);
-		}
-		elsif ($macro =~ /^\w+$/) {
-			$sub = $Vend::Cfg->{Sub}{$macro} || $Global::GlobalSub->{$macro};
-			$sub->();
-		}
+		run_macro($macro);
 #show_times("end AutoEnd macro") if $Global::ShowTimes;
 	}
   }
