@@ -1,8 +1,8 @@
 # Vend::File - Interchange file functions
 #
-# $Id: File.pm,v 2.14 2003/12/03 17:04:26 jon Exp $
+# $Id: File.pm,v 2.21 2005/11/08 18:14:45 jon Exp $
 # 
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -51,9 +51,11 @@ use Config;
 use Fcntl;
 use Errno;
 use Vend::Util;
+use File::Path;
+use File::Copy;
 use subs qw(logError logGlobal);
 use vars qw($VERSION @EXPORT @EXPORT_OK $errstr);
-$VERSION = substr(q$Revision: 2.14 $, 10);
+$VERSION = substr(q$Revision: 2.21 $, 10);
 
 sub writefile {
     my($file, $data, $opt) = @_;
@@ -232,7 +234,7 @@ sub flock_lock {
 
     if ($wait) {
 	my $trylimit = $Vend::Cfg->{Limit}{file_lock_retries} || 5;
-	my $failedcount;
+	my $failedcount = 0;
         while (
                 ! flock($fh, $flag)
                     and
@@ -359,7 +361,16 @@ sub get_filename {
 	my $i;
 	$levels = 1 unless defined $levels;
 	$chars = 1 unless defined $chars;
-	$dir = $Vend::Cfg->{ScratchDir} unless $dir;
+
+	# Accomodate PermanentDir not existing in pre-5.3.1 catalogs
+	# Block is better than always doing -d test
+	if(! $dir) {
+		$dir = $Vend::Cfg->{ScratchDir};
+	}
+	else {
+		mkdir $dir, 0777 unless -d $dir;
+	}
+
     for($i = 0; $i < $levels; $i++) {
 		$dir .= "/";
 		$dir .= substr($file, $i * $chars, $chars);
@@ -530,7 +541,7 @@ my %intrinsic = (
 	ic_scratch => sub {
 					my ($fn, $checkpath, $write, $sub, $compare) = @_;
 					my $false = $sub =~ s/^!\s*//;
-					my $status	= length($compare)
+					my $status	= defined $compare && length($compare)
 								? ($::Scratch->{$sub} eq $compare)
 								: ($::Scratch->{$sub});
 					return ! $false if $status;
@@ -651,7 +662,7 @@ sub allowed_file {
 		absolute_or_relative($fn)
 		)
 	{
-		if($Vend::admin and ! $write and $fn =~ /^$Global::RunDir/) {
+		if($Vend::admin and ! $write and $fn =~ /^$Global::RunDir/ and $fn !~ $relpat) {
 			$status = 1;
 		}
 		else {
