@@ -1,6 +1,6 @@
 # Vend::Order - Interchange order routing routines
 #
-# $Id: Order.pm,v 2.64 2004/03/06 22:16:57 mheins Exp $
+# $Id: Order.pm,v 2.83.2.1 2005/12/07 17:15:49 kwalsh Exp $
 #
 # Copyright (C) 2002-2003 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
@@ -29,7 +29,7 @@
 package Vend::Order;
 require Exporter;
 
-$VERSION = substr(q$Revision: 2.64 $, 10);
+$VERSION = substr(q$Revision: 2.83.2.1 $, 10);
 
 @ISA = qw(Exporter);
 
@@ -58,6 +58,7 @@ use Vend::Data;
 use Text::ParseWords;
 use Errno qw/:POSIX/;
 use strict;
+no warnings qw(uninitialized numeric);
 
 use autouse 'Vend::Error' => qw/do_lockout/;
 
@@ -126,166 +127,6 @@ my %Parse = (
 								$params =~ s/\s+//g;
 								return $params;
 							},
-	'length'		=>  sub {
-							my($name, $value, $msg) = @_;
-							$msg =~ s/^(\d+)(?:\s*-(\d+))?\s*//
-								or return undef;
-							my $min = $1;
-							my $max = $2;
-							my $len = length($value);
-
-							if($len < $min) {
-								$msg = errmsg(
-										"%s length %s less than minimum length %s.",
-										$name,
-										$len,
-										$min) if ! $msg;
-								return(0, $name, $msg);
-							}
-							elsif($max and $len > $max) {
-								$msg = errmsg(
-										"%s length %s more than maximum length %s.",
-										$name,
-										$len,
-										$max) if ! $msg;
-								return(0, $name, $msg);
-							}
-							return (1, $name, '');
-						},
-	'filter'			=> sub {		
-							my($name, $value, $code) = @_;
-							my $message;
-							my $filter;
-
-							$code =~ s/\\/\\\\/g;
-							if($code =~ /(["']).+?\1$/) {
-								my @code = Text::ParseWords::shellwords($code);
-								$message = pop(@code);
-								$filter = join " ", @code;
-							}
-							else {
-								($filter, $message) = split /\s+/, $code, 2;
-							}
-
-							my $test = Vend::Interpolate::filter_value($filter, $value, $name);
-							if($test ne $value) {
-								$message ||= errmsg("%s caught by filter %s", $name, $filter);
-								return ( 0, $name, $message);
-							}
-							return (1, $name, '');
-						},
-	'regex'			=>	sub {		
-							my($name, $value, $code) = @_;
-							my $message;
-
-							$code =~ s/\\/\\\\/g;
-							my @code = Text::ParseWords::shellwords($code);
-							if($code =~ /(["']).+?\1$/) {
-								$message = pop(@code);
-							}
-
-							for(@code) {
-								my $negate;
-								s/^!\s*// and $negate = 1;
-								my $op = $negate ? "!~" :  '=~';
-								my $regex = qr($_);
-								my $status;
-								if($negate) {
-									$status = ($value !~ $regex);
-								}
-								else {
-									$status = ($value =~ $regex);
-								}
-								if(! $status) {
-									$message = errmsg(
-										"failed pattern - %s",
-										"'$value' $op $_"
-										) if ! $message;
-									return ( 0, $name, $message);
-								}
-							}
-							return (1, $name, '');
-						},
-	'exists'			=> sub {
-							my($name, $value, $code) = @_;
-
-							$code =~ s/(\w+)(:+(\w+))?\s*//;
-							my $tab = $1
-								or return (0, $name, errmsg("no table specified"));
-							my $col = $3;
-							my $msg = $code;
-
-							my $db = database_exists_ref($tab)
-								or do {
-									$msg = errmsg(
-										"Table %s doesn't exist",
-										$tab,
-									);
-									return(0, $name, $msg);
-								};
-							my $used;
-							if(! $col) {
-								$used = $db->record_exists($value);
-							}
-							else {
-#::logDebug("Doing foreign key check, tab=$tab col=$col value=$value");
-								$used = $db->foreign($value, $col);
-							}
-
-#::logDebug("Checking exists, tab=$tab col=$col, used=$used");
-							if($used) {
-								return (1, $name, '');
-							}
-							else {
-								$msg = errmsg(
-										"Key %s does not exist in %s, try again.",
-										$value,
-										$tab,
-									) unless $msg;
-								return(0, $name, $msg);
-							}
-
-						},
-	'unique'			=> sub {
-							my($name, $value, $code) = @_;
-
-							$code =~ s/(\w+)(:+(\w+))?\s*//;
-							my $tab = $1
-								or return (0, $name, errmsg("no table specified"));
-							my $col = $3;
-							my $msg = $code;
-
-							my $db = database_exists_ref($tab)
-								or do {
-									$msg = errmsg(
-										"Table %s doesn't exist",
-										$tab,
-									);
-									return(0, $name, $msg);
-								};
-							my $used;
-							if(! $col) {
-								$used = $db->record_exists($value);
-							}
-							else {
-#::logDebug("Doing foreign key check, tab=$tab col=$col value=$value");
-								$used = $db->foreign($value, $col);
-							}
-
-#::logDebug("Checking unique, tab=$tab col=$col, used=$used");
-							if(! $used) {
-								return (1, $name, '');
-							}
-							else {
-								$msg = errmsg(
-										"Key %s already exists in %s, try again.",
-										$value,
-										$tab,
-									) unless $msg;
-								return(0, $name, $msg);
-							}
-
-						},
 	'&set'			=>	sub {		
 								my($ref,$params) = @_;
 								my ($var, $value) = split /\s+/, $params, 2;
@@ -298,38 +139,6 @@ my %Parse = (
 								my $msg = errmsg("%s set failed.", $var);
 								return ($value, $var, $msg);
 							},
-	future => sub {
-							my($name, $value, $code) = @_;
-							my $message;
-
-							my @code = Text::ParseWords::shellwords($code);
-							if($code =~ /(["']).+?\1$/) {
-								$message = pop(@code);
-							}
-							my $adjust = join " ", @code;
-							if(! $message) {
-								$message = errmsg(
-											"Date must be in the future at least %s",
-											$adjust,
-											);
-							}
-							if($value =~ /\0/) {
-								$value = Vend::Interpolate::filter_value(
-											'date_change',
-											$value,
-										);
-							}
-							my $current = Vend::Interpolate::mvtime(
-													undef,
-													{ adjust => $adjust },
-													"%Y%m%d%H%M",
-													);
-#::logDebug("current time: $current input value=$value");
-							if($value lt $current) {
-								return (0, $name, $message);
-							}
-							return (1, $name, '');
-						},
 );
 
 sub _update {
@@ -367,11 +176,12 @@ sub _format {
 	}
 	elsif ($OrderCheck and $sub = $OrderCheck->{$routine}) {
 #::logDebug("Using coderef OrderCheck = $sub");
-		@args = ($ref,$var,$val);
+		@args = ($ref,$var,$val,$message);
+		undef $message;
 	}
 	elsif (defined &{"_$routine"}) {
 		$sub = \&{"_$routine"};
-		@args = ($ref,$var,$val);
+		@args = ($ref,$var,$val,$message);
 	}
 	else {
 		return (undef, $var, errmsg("No format check routine for '%s'", $routine));
@@ -419,6 +229,11 @@ sub _and_check {
 	return chain_checks(0, @_);
 }
 
+sub _always_pass {
+	my($ref,$var,$val) = @_;
+	return (1, $var, '')
+}
+
 sub _or_check {
 	if(! length($_[1]) ) {
 		$And = 0;
@@ -623,6 +438,13 @@ sub build_cc_info {
 		return;
 	}
 
+	if(my $num = $cardinfo->{MV_CREDIT_CARD_NUMBER}) {
+		my @quads;
+		$num =~ s/\D//g;
+		@quads = $num =~ m{(\d\d\d\d)(\d\d\d\d)(\d\d\d\d)(\d+)};
+		$cardinfo->{MV_CREDIT_CARD_QUADS} = join "-", @quads;
+	}
+
 	$template = $template ||
 		$::Variable->{MV_CREDIT_CARD_INFO_TEMPLATE} ||
 		join("\t", qw(
@@ -643,20 +465,35 @@ sub guess_cc_type {
 	my ($ccnum) = @_;
 	$ccnum =~ s/\D+//g;
 
+	if(my $subname = $Vend::Cfg->{SpecialSub}{guess_cc_type}) {
+		my $sub = $Vend::Cfg->{Sub}{$subname} || $Global::GlobalSub->{$subname};
+		my $guess;
+		if( $sub and $guess = $sub->($ccnum) ) {
+			return $guess;
+		}
+	}
+
 	# based on logic by Karl Moore from http://www.vb-world.net/tips/tip509.html
 	if ($ccnum eq '')										{ '' }
-	elsif ($ccnum =~ /^4(?:\d{12}|\d{15})$/)				{ 'visa' }
-	elsif ($ccnum =~ /^5[1-5]\d{14}$/)						{ 'mc' }
-	elsif ($ccnum =~ /^6011\d{12}$/)						{ 'discover' }
-	elsif ($ccnum =~ /^3[47]\d{13}$/)						{ 'amex' }
-	elsif ($ccnum =~ /^3(?:6\d{12}|0[0-5]\d{11})$/)			{ 'dinersclub' }
-	elsif ($ccnum =~ /^38\d{12}$/)							{ 'carteblanche' }
-	elsif ($ccnum =~ /^2(?:014|149)\d{11}$/)				{ 'enroute' }
-	elsif ($ccnum =~ /^56\d{14}$/)                          { 'bankcard' }
-	elsif ($ccnum =~ /^(?:3\d{15}|2131\d{11}|1800\d{11})$/)	{ 'jcb' }
-	else													{ 'other' }
-}
+	elsif ($ccnum =~ /^4(?:\d{12}|\d{15})$/)				{ return 'visa' }
+	elsif ($ccnum =~ /^5[1-5]\d{14}$/)						{ return 'mc' }
+	elsif ($ccnum =~ /^6011\d{12}$/)						{ return 'discover' }
+	elsif ($ccnum =~ /^3[47]\d{10,13}$/)						{ return 'amex' }
+	elsif ($ccnum =~ /^3(?:6\d{12}|0[0-5]\d{11})$/)			{ return 'dinersclub' }
+	elsif ($ccnum =~ /^38\d{12}$/)							{ return 'carteblanche' }
+	elsif ($ccnum =~ /^2(?:014|149)\d{11}$/)				{ return 'enroute' }
+	elsif ($ccnum =~ /^(?:3\d{15}|2131\d{11}|1800\d{11})$/)	{ return 'jcb' }
+    elsif ($ccnum =~ /^49(03(0[2-9]|3[5-9])|11(0[1-2]|7[4-9]|8[1-2])|36[0-9]{2})\d{10}(\d{2,3})?$/
+      or $ccnum =~ /^564182\d{10}(\d{2,3})?$/
+      or $ccnum =~ /^6(3(33[0-4][0-9])|759[0-9]{2})\d{10}(\d{2,3})?$/) 
+															{ return 'switch' }
 
+	elsif ($ccnum =~ /^56\d{14}$/)                          { return 'bankcard' }
+    elsif ($ccnum =~ /^6(3(34[5-9][0-9])|767[0-9]{2})\d{10}(\d{2,3})?$/)
+		 { return 'solo' }
+
+	else { return $::Variable->{MV_PAYMENT_OTHER_CARD} || 'other' }
+}
 
 # Takes a reference to a hash (usually %CGI::values) that contains
 # the following:
@@ -1023,6 +860,7 @@ sub check_order {
 		undef $individual;
 	}
 
+#::logDebug("nextpage=$CGI::values{mv_nextpage}");
 	for my $profile (split /\0+/, $profiles) {
 
 		$status = check_order_each($profile, $vref, $individual);
@@ -1054,6 +892,7 @@ sub check_order {
 			}
 		}
 		else {
+#::logDebug("Got to status=$status on profile=$profile");
 			if($Fail_page) {
 				$np = $CGI::values{mv_nextpage} = $Fail_page;
 			}
@@ -1094,6 +933,7 @@ sub check_order {
 	my $errors = join "\n", @Errors;
 #::logDebug("Errors after checking profile(s):\n$errors") if $errors;
 	$errors = '' unless defined $errors and ! $Success;
+#::logDebug("status=$status nextpage=$CGI::values{mv_nextpage}");
 	return ($status, $Final, $errors);
 }
 
@@ -1457,18 +1297,6 @@ sub _email {
 	}
 }
 
-sub _email_only {
-	my($ref, $var, $val) = @_;
-	if($val and $val =~ /^[\040-\176]+\@[-A-Za-z0-9.]+\.[A-Za-z]+$/) {
-		return (1, $var, '');
-	}
-	else {
-		return (undef, $var,
-			errmsg( "'%s' not an email address", $val )
-		);
-	}
-}
-
 # Contributed by Ton Verhagen -- April 15, 2000
 sub _isbn {
 	# $ref is to $::Values hash (well, actually ref to %CGI::values)
@@ -1482,14 +1310,24 @@ sub _isbn {
 	# Values (product)	0 +81 +32 + 0 + 0 + 5 +24 + 9 + 6 + 8 --> sum is: 165
 	# Sum must be divisable by 11 without remainder: 165/11=15 (no remainder)
 	# Result: isbn 0-940016-33-8 is a valid isbn number.
+	# Note: the last "digit" could be a "X", which would be treated as 10 in the above
 	
 	my($ref, $var, $val) = @_;
-	$val =~ s/\D//g;	# weed out non-digits
+	$val =~ s/[^\dXx]//g;	# weed out non-digits
 	if( $val && length($val) == 10 ) {
 	  my @digits = split("", $val);
 	  my $sum=0;
 	  for(my $i=10; $i > 0; $i--) {
-		$sum += $digits[10-$i] * $i;
+	  	my $d = $digits[10 - $i];
+		if ($d =~ /[Xx]/) {
+		    if ($i == 1) {
+			$d = 10;
+		    }
+		    else {
+			return (undef, $var, errmsg("'%s' not a valid isbn number", $val));
+		    }
+		}
+		$sum += $d * $i;
 	  }
 	  return ( $sum%11 ? 0 : 1, $var, '' );
 	}
@@ -2047,7 +1885,6 @@ sub route_order {
 
 	foreach $msg (@out) {
 		eval {
-#### change this to use Vend::Mail::send
 			send_mail(@$msg);
 		};
 		if($@) {
@@ -2206,15 +2043,20 @@ my @Scan_modifiers = qw/
 sub update_quantity {
     return 1 unless defined  $CGI::values{"quantity0"}
 		|| $CGI::values{mv_quantity_update};
-	my($h, $i, $quantity, $modifier, $cart);
+	my ($h, $i, $quantity, $modifier, $cart, $cartname, %altered_items, %old_items);
 
 	if ($CGI::values{mv_cartname}) {
-		$cart = $::Carts->{$CGI::values{mv_cartname}} ||= [];
+		$cart = $::Carts->{$cartname = $CGI::values{mv_cartname}} ||= [];
 	}
 	else {
 		$cart = $Vend::Items;
+		$cartname = $Vend::CurrentCart;
 	}
 
+	my ($raise_event, $quantity_raise_event)
+		= @{$Vend::Cfg}{qw/CartTrigger CartTriggerQuantity/};
+	$quantity_raise_event = $raise_event && $quantity_raise_event;
+
 	my @mods;
 	@mods = @{$Vend::Cfg->{UseModifier}} if $Vend::Cfg->{UseModifier};
 
@@ -2234,6 +2076,7 @@ sub update_quantity {
 			next if
 				!   defined $CGI::values{"$h$i"}
 				and defined $cart->[$i]{$h};
+			$old_items{$i} ||= { %{$cart->[$i]} } if $raise_event;
 			$modifier = $CGI::values{"$h$i"}
 					  || (defined $cart->[$i]{$h} ? '' : undef);
 #::logDebug("line $i modifier $h now $modifier");
@@ -2242,6 +2085,9 @@ sub update_quantity {
 				$modifier =~ s/\0$//;
 				$modifier =~ s/^\0//;
 				$modifier =~ s/\0/, /g;
+				$altered_items{$i} = 1
+					if $raise_event
+					and $cart->[$i]->{$h} ne $modifier;
 				$cart->[$i]->{$h} = $modifier;
 				$::Values->{"$h$i"} = $modifier;
 				delete $CGI::values{"$h$i"};
@@ -2256,14 +2102,21 @@ sub update_quantity {
     	$quantity = $CGI::values{"quantity$i"};
     	next unless defined $quantity;
 		my $do_update;
+		my $old_item = $old_items{$i} ||= { %$line } if $raise_event;
     	if ($quantity =~ m/^\d*$/) {
         	$line->{'quantity'} = $quantity || 0;
 			$do_update = 1;
+			$altered_items{$i} = 1
+				if $quantity_raise_event
+				and $line->{quantity} != $old_item->{quantity};
     	}
     	elsif ($quantity =~ m/^[\d.]+$/
 				and $Vend::Cfg->{FractionalItems} ) {
         	$line->{'quantity'} = $quantity;
 			$do_update = 1;
+			$altered_items{$i} = 1
+				if $quantity_raise_event
+				and $line->{quantity} != $old_item->{quantity};
     	}
 		# This allows a last-positioned input of item quantity to
 		# remove the item
@@ -2345,9 +2198,17 @@ sub update_quantity {
 				else {
 					$line->{code}	= $sku;
 				}
+				$altered_items{$i} = 1 if $raise_event;
 			}
 		}
-    }
+	}
+
+	Vend::Cart::trigger_update(
+			$cart,
+			$cart->[$_], # new item version
+			$old_items{$_}, # old item version
+			$cartname
+		) for sort { $a <=> $b } keys %altered_items;
 #::logDebug("after update, cart is: " . ::uneval($cart));
 
 	# If the user has put in "0" for any quantity, delete that item
@@ -2382,14 +2243,19 @@ sub add_items {
 
 	::update_quantity() if ! defined $CGI::values{mv_orderline};
 
-	my $cart;
-	if($CGI::values{mv_cartname}) {
-		$cart = $::Carts->{$CGI::values{mv_cartname}} ||= [];
+	my ($cart, $cartname);
+	if ($cartname = $CGI::values{mv_cartname}) {
+		$cart = $::Carts->{$cartname} ||= [];
 	}
 	else {
 		$cart = $Vend::Items;
+		$cartname = $Vend::CurrentCart;
 	}
 
+	my ($raise_event,$track_quantity)
+		= @{$Vend::Cfg}{qw/CartTrigger CartTriggerQuantity/};
+	$raise_event = @$raise_event if ref $raise_event eq 'ARRAY';
+
 	@items      = split /\0/, ($items);
 	@quantities = split /\0/, ($quantities || delete $CGI::values{mv_order_quantity} || '');
 	@bases      = split /\0/, delete $CGI::values{mv_order_mv_ib}
@@ -2454,9 +2320,18 @@ sub add_items {
 	}
 
 	@group   = split /\0/, (delete $CGI::values{mv_order_group} || '');
+	
+	my $inc;
 	for( my $i = 0; $i < @group; $i++ ) {
 #::logDebug("processing order group=$group[$i]");
-	   $attr{mv_mi}->[$i] = $group[$i] ? ++$Vend::Session->{pageCount} : 0;
+		if($group[$i]) {
+			$inc ||= time();
+			my $add = sprintf('%06d', ++$Vend::Session->{pageCount});
+			$attr{mv_mi}->[$i] = $inc . $add;
+		}
+		else {
+			$attr{mv_mi}->[$i] = 0;
+		}
 	}
 
 	$j = 0;
@@ -2512,11 +2387,19 @@ sub add_items {
 			foreach $i (0 .. $#$cart) {
 				if ($cart->[$i]->{'code'} eq $code) {
 					next unless $base eq $cart->[$i]->{mv_ib};
+					next if $cart->[$i]->{mv_free_item};
 					$found = $i;
 					# Increment quantity. This is different than
 					# the standard handling because we are ordering
 					# accessories, and may want more than 1 of each
+					my %old_item = %{$cart->[$i]} if $raise_event and $track_quantity;
 					$cart->[$i]{quantity} = $set ? $quantity : $cart->[$i]{quantity} + $quantity;
+					Vend::Cart::trigger_update(
+							$cart,
+							$cart->[$i], # new row
+							\%old_item, # old row
+							$cartname
+						) if $raise_event and $track_quantity;
 				}
 			}
 		} # INCREMENT
@@ -2553,14 +2436,33 @@ sub add_items {
 			}
 			if($Vend::Cfg->{AutoModifier}) {
 				foreach $i (@{$Vend::Cfg->{AutoModifier}}) {
-					my ($table,$key) = split /:/, $i;
+					my $attr;
+					my ($table,$key,$foreign) = split /:+/, $i, 3;
+
+					if($table =~ /=/) {
+						($attr, $table) = split /\s*=\s*/, $table, 2;
+					}
+
+					if(! $key and ! $foreign) {
+						$attr ||= $table;
+						$item->{$attr} = item_common($item, $table);
+						next;
+					}
+
 					unless ($key) {
 						$key = $table;
-						$item->{$key} = item_common($item, $key, $code)
-					}
-					else {
-						$item->{$key} = tag_data($table, $key, $code);
+						$table = $item->{mv_ib};
 					}
+
+					$attr ||= $key;
+
+
+					my $select = $foreign ? $item->{$foreign} : $code;
+					$select ||= $code;
+
+#::logDebug("attr=$attr table=$table key=$key select=$select foreign=$foreign");
+					$item->{$attr} = ::tag_data($table, $key, $select);
+#::logDebug("item->$attr=$item->{$attr}");
 				}
 			}
 
@@ -2586,13 +2488,25 @@ sub add_items {
 			}
 
 			if($lines[$j] =~ /^\d+$/ and defined $cart->[$lines[$j]] ) {
+				my %old = %{$cart->[$lines[$j]]} if $raise_event;
 				$cart->[$lines[$j]] = $item;
+				Vend::Cart::trigger_update(
+						$cart,
+						$item, # new item
+						\%old, # old item
+						$cartname,
+					) if $raise_event;
 			}
 			else {
 # TRACK
 				$Vend::Track->add_item($cart,$item) if $Vend::Track;
 # END TRACK
 				push @$cart, $item;
+				Vend::Cart::trigger_add(
+						$cart,
+						$item, # new item
+						$cartname,
+					) if $raise_event;
 			}
 		}
 		$j++;
