@@ -1,8 +1,8 @@
 # Vend::Parse - Parse Interchange tags
 # 
-# $Id: Parse.pm,v 2.29 2004/02/11 14:34:33 jon Exp $
+# $Id: Parse.pm,v 2.37 2005/11/08 18:14:45 jon Exp $
 #
-# Copyright (C) 2002-2003 Interchange Development Group
+# Copyright (C) 2002-2005 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
 #
 # This program was originally based on Vend 0.2 and 0.3
@@ -36,12 +36,13 @@ require Exporter;
 
 @ISA = qw(Exporter Vend::Parser);
 
-$VERSION = substr(q$Revision: 2.29 $, 10);
+$VERSION = substr(q$Revision: 2.37 $, 10);
 
 @EXPORT = ();
 @EXPORT_OK = qw(find_matching_end);
 
 use strict;
+no warnings qw(uninitialized numeric);
 
 use vars qw($VERSION);
 
@@ -187,8 +188,10 @@ $Routine{restrict} = sub {
 		$default = 1;
 		$opt->{policy} = 'deny';
 	}
-	my @enable  = split /[\s,\0]+/, $enable;
-	my @disable = split /[\s,\0]+/, $opt->{disable};
+	my @enable;
+	my @disable;
+	$enable			and @enable  = split /[\s,\0]+/, $enable;
+	$opt->{disable} and @disable = split /[\s,\0]+/, $opt->{disable};
 
 	for(@enable, @disable) {
 		$_ = lc $_;
@@ -260,7 +263,6 @@ my %Alias = (
 				),
 					getlocale		=> 'setlocale get=1',
 					process_search		=> 'area href=search',
-					buzzard		=> 'data table=products column=artist key=',
 			);
 
 my %Interpolate = (
@@ -418,17 +420,27 @@ sub do_tag {
 
 	}
 
-	die errmsg("Unauthorized for admin tag %s", $tag)
-		if defined $Vend::Cfg->{AdminSub}{$tag} and
-			($Vend::restricted or ! $Vend::admin);
-	
+	if (! defined $Routine{$tag} and $Global::AccumulateCode) {
+#::logDebug("missing $tag, trying code_from_file");
+		if($Alias{$tag}) {
+			$tag = $Alias{$tag};
+#::logDebug("missing $tag found alias=$tag");
+		}
+		else {
+			$Routine{$tag} = Vend::Config::code_from_file('UserTag', $tag)
+				if ! $Routine{$tag};
+		}
+	}
+
 	if (! defined $Routine{$tag}) {
+#::logDebug("missing $tag, but didn't try code_from_file?");
         if (! $Alias{$tag}) {
             ::logError("Tag '$tag' not defined.");
             return undef;
         }
         $tag = $Alias{$tag};
 	};
+
 	if($Special{$tag}) {
 		my $ref = pop(@_);
 		my @args = @$ref{ @{$Order{$tag}} };
@@ -450,6 +462,7 @@ sub do_tag {
 		return &{$Routine{$tag}}(@args, $text || undef);
 	}
 	else {
+#::logDebug("Parse-do_tag tag=$tag: args now=" . ::uneval_it(\@_) );
 		return &{$Routine{$tag}}(@_);
 	}
 }
@@ -457,6 +470,11 @@ sub do_tag {
 sub resolve_args {
 	my $tag = shift;
 #::logDebug("resolving args for $tag, attrAlias = $attrAlias{$tag}");
+	if (! defined $Routine{$tag} and $Global::AccumulateCode) {
+#::logDebug("missing $tag, trying code_from_file");
+		$Routine{$tag} = Vend::Config::code_from_file('UserTag', $tag);
+	}
+
 	return @_ unless defined $Routine{$tag};
 	my $ref = shift;
 	my @list;
@@ -627,6 +645,18 @@ sub start {
 
     # $attr is reference to a HASH, $attrseq is reference to an ARRAY
 	my $aliasname = '';
+	if (! defined $Routine{$tag} and $Global::AccumulateCode) {
+		my $newtag;
+		if($newtag = $Alias{$tag}) {
+			$newtag =~ s/\s+.*//s;
+			Vend::Config::code_from_file('UserTag', $newtag)
+				unless $Routine{$newtag};
+		}
+		else {
+			Vend::Config::code_from_file('UserTag', $tag);
+		}
+	}
+
 	unless (defined $Routine{$tag}) {
 		if(defined $Alias{$tag}) {
 			$aliasname = $tag;
@@ -679,10 +709,15 @@ sub start {
 
 	$attr->{enable_html} = 1 if $Vend::Cfg->{Promiscuous};
 	$attr->{reparse} = 1
-		unless (defined $NoReparse{$tag} || defined $attr->{'reparse'});
+		unless (
+			defined $NoReparse{$tag}
+			|| defined $attr->{reparse}
+			|| $::Pragma->{no_default_reparse}
+		);
 
 	my ($routine,@args);
 
+#::logDebug("tag=$tag order=$Order{$tag}");
 	# Check for old-style positional tag
 	if(!@$attrseq and $origtext =~ s/\[[-\w]+\s+//i) {
 			$origtext =~ s/\]$//;
