@@ -1,8 +1,8 @@
 # Vend::Payment::Linkpoint - Interchange Linkpoint support
 #
-# $Id: Linkpoint.pm,v 1.5 2005/11/08 18:14:46 jon Exp $
+# $Id: Linkpoint.pm,v 1.5.2.3 2006/05/19 14:48:48 jon Exp $
 #
-# Copyright (C) 2002-2005 Interchange Development Group
+# Copyright (C) 2002-2006 Interchange Development Group
 # Copyright (C) 2002 Stefan Hornburg (Racke) <racke@linuxia.de>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -108,6 +108,46 @@ The type of transaction to be run. Valid values are:
 
 Default is C<sale>.
 
+=item check_sub
+
+Name of a Sub or GlobalSub to be called after the result hash has been 
+received from LinkPoint. A reference to the modifiable result hash is
+passed into the subroutine, and it should return true (in the Perl truth
+sense) if its checks were successful, or false if not.
+
+This can come in handy since LinkPoint has no option to decline a charge
+when AVS data come back negative. 
+
+If you want to fail based on a bad AVS check, make sure you're only
+doing an auth -- B<not a sale>, or your customers would get charged on
+orders that fail the AVS check and never get logged in your system!
+
+Add the parameters like this:
+
+	Route  linkpoint  check_sub  avs_check
+
+This is a matching sample subroutine you could put in interchange.cfg:
+			
+	GlobalSub <<EOR
+	sub avs_check {
+		my ($result) = @_;
+		my $avs = $result->{r_avs};
+		my ($addr, $zip) = split //, $avs;
+		return 1 if $addr eq 'Y' or $zip eq 'Y';
+		return 1 if $addr eq 'X' and $zip eq 'X';
+		$result->{MStatus} = 'failure';
+		$result->{r_error} = $result->{MErrMsg} = "The billing address you entered does not match the cardholder's billing address";
+		return 0; 
+	}
+	EOR
+
+That would work equally well as a Sub in catalog.cfg. It will succeed if
+either the address or zip is 'Y', or if both are unknown. If it fails,
+it sets the error message in the result hash.
+
+Of course you can use this sub to do any other post-processing you
+want as well.
+
 =back
 
 =head2 Troubleshooting
@@ -251,7 +291,10 @@ sub linkpoint {
 	$actual->{mv_credit_card_exp_year} =~ s/\D//g;
 	$actual->{mv_credit_card_exp_year} =~ s/\d\d(\d\d)/$1/;
 	$actual->{mv_credit_card_number} =~ s/\D//g;
-	$actual->{b_zip} =~ s/\D//g;
+	if($actual->{b_country} eq 'US') {
+		$actual->{b_zip} =~ s/\D//g;
+		$actual->{b_zip} = substr($actual->{b_zip}, 0, 5);
+	}
 
 	my $exp = sprintf '%02d%02d', $actual->{mv_credit_card_exp_month},
 		$actual->{mv_credit_card_exp_year};
@@ -264,6 +307,7 @@ sub linkpoint {
 		mauthcapture 	=>	'SALE',
 		mauthonly		=>	'PREAUTH',
 		sale	 		=>	'SALE',
+		settle_prior	=>	'POSTAUTH',
 	);
 	
 	if (defined $type_map{$transtype}) {
@@ -293,6 +337,22 @@ sub linkpoint {
 	$scompany =~ s/\&/ /g;
 	$bcompany =~ s/\&/ /g;
 	
+	my %delmap = (
+		POSTAUTH => [ 
+					qw(
+						shipping
+						subtotal
+						tax
+						vattax
+						cardnumber
+						cardexpmonth
+						cardexpyear
+						addrnum
+						company
+					)
+			],
+	);
+
 	my %varmap = ( qw /
 				   name b_name
 				   address1 b_address1
@@ -317,6 +377,7 @@ sub linkpoint {
 		 port => '1129',
 		 configfile => $user,
 		 keyfile => $keyfile,
+		 ordertype => $transtype,
 		 result => 'LIVE',
 		 terminaltype => 'UNSPECIFIED',
 		 shipping => $shipping,
@@ -331,11 +392,18 @@ sub linkpoint {
 		 debbugging => $opt->{debuglevel},
 		 company => $bcompany,
 		 scompany => $scompany, # API is broken for Shipping Company per Linkpoint support
+		 oid => $order_id,
 		);
 
     for (keys %varmap) {
         $query{$_} = $actual->{$varmap{$_}};
     }
+
+	if(my $map = $delmap{$transtype}) {
+		for(@$map) {
+			delete $query{$_};
+		}
+	}
     
     $query{saddress2} = $actual->{address1} . ' ' . $actual->{address2}; # API is broken for Shipping Address Line 1, put line 1 and line 2 in API line 2 per Linkpoint support
 	
@@ -368,10 +436,30 @@ sub linkpoint {
 			if defined $result{$result_map{$_}};
 	}
 
+	my $approve;
 	if ($result{'r_approved'} eq "APPROVED") {
+		if (my $check_sub_name = $opt->{check_sub} || charge_param('check_sub')) {
+			my $check_sub = $Vend::Cfg->{Sub}{$check_sub_name}
+							|| $Global::GlobalSub->{$check_sub_name};
+
+			if (ref $check_sub eq 'CODE') {
+				$approve = $check_sub->(\%result);
+			}
+			else {
+				logError(__PACKAGE__ . ": non-existent check_sub routine %s.", $check_sub_name);
+			}
+		}
+		else {
+			$approve = 1;
+		}
+	}
+
+
+	if($approve) {
 		$result{'MStatus'} = 'success';
 		$result{'MErrMsg'} = $result{'r_code'};
-	} else {
+	}
+	else {
 		my $msg = errmsg("Charge error: %s Please call in your order or try again.",
 			$result{'r_error'},
 		);
@@ -388,3 +476,4 @@ sub linkpoint {
 package Vend::Payment::Linkpoint;
 
 1;
+
